@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -207,11 +206,12 @@ static void at_call_barring_remove(struct ofono_call_barring *cb)
 {
 	GAtChat *chat = ofono_call_barring_get_data(cb);
 
+	g_idle_remove_by_data(cb);
 	g_at_chat_unref(chat);
 	ofono_call_barring_set_data(cb, NULL);
 }
 
-static struct ofono_call_barring_driver driver = {
+static const struct ofono_call_barring_driver driver = {
 	.name		= "atmodem",
 	.probe		= at_call_barring_probe,
 	.remove		= at_call_barring_remove,
