@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -101,7 +100,7 @@ static void calypso_stk_envelope(struct ofono_stk *stk, int length,
 
 	DBG("");
 
-	if (cbd == NULL || buf == NULL)
+	if (buf == NULL)
 		goto error;
 
 	len = sprintf(buf, "AT%%SATE=\"");
@@ -148,7 +147,7 @@ static void calypso_stk_terminal_response(struct ofono_stk *stk, int length,
 
 	DBG("");
 
-	if (cbd == NULL || buf == NULL)
+	if (buf == NULL)
 		goto error;
 
 	len = sprintf(buf, "AT%%SATR=\"");
@@ -292,7 +291,7 @@ static void calypso_stk_remove(struct ofono_stk *stk)
 	g_free(sd);
 }
 
-static struct ofono_stk_driver driver = {
+static const struct ofono_stk_driver driver = {
 	.name			= "calypsomodem",
 	.probe			= calypso_stk_probe,
 	.remove			= calypso_stk_remove,
