@@ -223,23 +223,11 @@ static gboolean gprs_context_string_to_type(const char *str,
 	return FALSE;
 }
 
-static gboolean assign_context(struct pri_context *ctx, unsigned int use_cid)
+static struct ofono_gprs_context *find_avail_gprs_context(
+							struct pri_context *ctx)
 {
-	struct l_uintset *used_cids = ctx->gprs->used_cids;
 	GSList *l;
 
-	if (used_cids == NULL)
-		return FALSE;
-
-	if (!use_cid)
-		use_cid = l_uintset_find_unused_min(used_cids);
-
-	if (use_cid > l_uintset_get_max(used_cids))
-		return FALSE;
-
-	l_uintset_put(used_cids, use_cid);
-	ctx->context.cid = use_cid;
-
 	for (l = ctx->gprs->context_drivers; l; l = l->next) {
 		struct ofono_gprs_context *gc = l->data;
 
@@ -257,24 +245,45 @@ static gboolean assign_context(struct pri_context *ctx, unsigned int use_cid)
 				gc->type != ctx->type)
 			continue;
 
-		ctx->context_driver = gc;
-		ctx->context_driver->inuse = TRUE;
-
-		if (ctx->context.proto == OFONO_GPRS_PROTO_IPV4V6 ||
-				ctx->context.proto == OFONO_GPRS_PROTO_IP)
-			gc->settings->ipv4 = g_new0(struct ipv4_settings, 1);
-
-		if (ctx->context.proto == OFONO_GPRS_PROTO_IPV4V6 ||
-				ctx->context.proto == OFONO_GPRS_PROTO_IPV6)
-			gc->settings->ipv6 = g_new0(struct ipv6_settings, 1);
-
-		return TRUE;
+		return gc;
 	}
 
-	l_uintset_take(used_cids, ctx->context.cid);
-	ctx->context.cid = 0;
+	return NULL;
+}
 
-	return FALSE;
+static gboolean assign_context(struct pri_context *ctx, unsigned int use_cid)
+{
+	struct l_uintset *used_cids = ctx->gprs->used_cids;
+	struct ofono_gprs_context *gc;
+
+	if (used_cids == NULL)
+		return FALSE;
+
+	if (!use_cid)
+		use_cid = l_uintset_find_unused_min(used_cids);
+
+	if (use_cid > l_uintset_get_max(used_cids))
+		return FALSE;
+
+	gc = find_avail_gprs_context(ctx);
+	if (gc == NULL)
+		return FALSE;
+
+	l_uintset_put(used_cids, use_cid);
+	ctx->context.cid = use_cid;
+
+	ctx->context_driver = gc;
+	ctx->context_driver->inuse = TRUE;
+
+	if (ctx->context.proto == OFONO_GPRS_PROTO_IPV4V6 ||
+			ctx->context.proto == OFONO_GPRS_PROTO_IP)
+		gc->settings->ipv4 = g_new0(struct ipv4_settings, 1);
+
+	if (ctx->context.proto == OFONO_GPRS_PROTO_IPV4V6 ||
+			ctx->context.proto == OFONO_GPRS_PROTO_IPV6)
+		gc->settings->ipv6 = g_new0(struct ipv6_settings, 1);
+
+	return TRUE;
 }
 
 static void release_context(struct pri_context *ctx)
@@ -1099,7 +1108,7 @@ static DBusMessage *pri_set_name(struct pri_context *ctx, DBusConnection *conn,
 	if (strlen(name) > MAX_CONTEXT_NAME_LENGTH)
 		return __ofono_error_invalid_format(msg);
 
-	if (ctx->name && g_str_equal(ctx->name, name))
+	if (g_str_equal(ctx->name, name))
 		return dbus_message_new_method_return(msg);
 
 	strcpy(ctx->name, name);
@@ -1556,6 +1565,19 @@ static bool have_read_settings(struct ofono_gprs *gprs)
 	return false;
 }
 
+static void pri_context_signal_active(struct pri_context *ctx)
+{
+	DBusConnection *conn;
+	dbus_bool_t value;
+
+	value = ctx->active;
+	conn = ofono_dbus_get_connection();
+
+	ofono_dbus_signal_property_changed(conn, ctx->path,
+					OFONO_CONNECTION_CONTEXT_INTERFACE,
+					"Active", DBUS_TYPE_BOOLEAN, &value);
+}
+
 static void detach_active_contexts(struct ofono_gprs *gprs)
 {
 	GSList *l;
@@ -1579,7 +1601,9 @@ static void detach_active_contexts(struct ofono_gprs *gprs)
 			gc->driver->detach_shutdown(gc, ctx->context.cid);
 
 		/* Make sure the context is properly cleared */
+		pri_reset_context_settings(ctx);
 		release_context(ctx);
+		pri_context_signal_active(ctx);
 	}
 }
 
@@ -2032,7 +2056,6 @@ void ofono_gprs_cid_activated(struct ofono_gprs  *gprs, unsigned int cid,
 
 	if (assign_context(pri_ctx, cid) == FALSE) {
 		ofono_warn("Can't assign context to driver for APN.");
-		release_context(pri_ctx);
 		return;
 	}
 
@@ -2235,8 +2258,6 @@ static void gprs_deactivate_for_all(const struct ofono_error *error,
 {
 	struct pri_context *ctx = data;
 	struct ofono_gprs *gprs = ctx->gprs;
-	DBusConnection *conn;
-	dbus_bool_t value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
 		__ofono_dbus_pending_reply(&gprs->pending,
@@ -2246,12 +2267,7 @@ static void gprs_deactivate_for_all(const struct ofono_error *error,
 
 	pri_reset_context_settings(ctx);
 	release_context(ctx);
-
-	value = ctx->active;
-	conn = ofono_dbus_get_connection();
-	ofono_dbus_signal_property_changed(conn, ctx->path,
-					OFONO_CONNECTION_CONTEXT_INTERFACE,
-					"Active", DBUS_TYPE_BOOLEAN, &value);
+	pri_context_signal_active(ctx);
 
 	gprs_deactivate_next(gprs);
 }
@@ -2730,14 +2746,15 @@ void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
 					unsigned int cid)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
+	struct ofono_gprs *gprs = gc->gprs;
 	GSList *l;
 	struct pri_context *ctx;
 	dbus_bool_t value;
 
-	if (gc->gprs == NULL)
+	if (gprs == NULL)
 		return;
 
-	for (l = gc->gprs->contexts; l; l = l->next) {
+	for (l = gprs->contexts; l; l = l->next) {
 		ctx = l->data;
 
 		if (ctx->context.cid != cid)
@@ -2759,10 +2776,13 @@ void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
 	 * If "Attached" property was about to be signalled as TRUE but there
 	 * were still active contexts, try again to signal "Attached" property
 	 * to registered applications after active contexts have been released.
+	 *
+	 * "Attached" could also change to FALSE in case of LTE and getting
+	 * deactivated
 	 */
-	if (gc->gprs->flags & GPRS_FLAG_ATTACHED_UPDATE) {
-		gc->gprs->flags &= ~GPRS_FLAG_ATTACHED_UPDATE;
-		gprs_attached_update(gc->gprs);
+	if (on_lte(gprs) || gprs->flags & GPRS_FLAG_ATTACHED_UPDATE) {
+		gprs->flags &= ~GPRS_FLAG_ATTACHED_UPDATE;
+		gprs_attached_update(gprs);
 	}
 }
 
