@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -53,8 +53,8 @@ static gboolean add_plugin(void *handle, struct ofono_plugin_desc *desc)
 		return FALSE;
 
 	if (g_str_equal(desc->version, OFONO_VERSION) == FALSE) {
-		ofono_error("Version mismatch for %s: found %s, expected %s",
-			    desc->description, desc->version, OFONO_VERSION);
+		ofono_error("Invalid version %s for %s", desc->version,
+							desc->description);
 		return FALSE;
 	}
 
@@ -66,6 +66,8 @@ static gboolean add_plugin(void *handle, struct ofono_plugin_desc *desc)
 	plugin->active = FALSE;
 	plugin->desc = desc;
 
+	__ofono_log_enable(desc->debug_start, desc->debug_stop);
+
 	plugins = g_slist_insert_sorted(plugins, plugin, compare_priority);
 
 	return TRUE;
@@ -188,17 +190,26 @@ void __ofono_plugin_cleanup(void)
 
 	DBG("");
 
+	/*
+	 * Terminate the plugins but don't unload the libraries yet.
+	 * Plugins may reference data structures allocated by each other.
+	 */
 	for (list = plugins; list; list = list->next) {
 		struct ofono_plugin *plugin = list->data;
 
 		if (plugin->active == TRUE && plugin->desc->exit)
 			plugin->desc->exit();
+	}
+
+	/* Second pass - unload the libraries */
+	for (list = plugins; list; list = list->next) {
+		struct ofono_plugin *plugin = list->data;
 
 		if (plugin->handle)
 			dlclose(plugin->handle);
-
-		g_free(plugin);
 	}
 
-	g_slist_free(plugins);
+	/* Finally, free the memory */
+	g_slist_free_full(plugins, g_free);
+	plugins = NULL;
 }
