@@ -44,11 +44,11 @@ static GSList *g_drivers = NULL;
 struct ofono_radio_settings {
 	DBusMessage *pending;
 	int flags;
-	enum ofono_radio_access_mode mode;
+	unsigned int mode;
 	enum ofono_radio_band_gsm band_gsm;
 	enum ofono_radio_band_umts band_umts;
 	ofono_bool_t fast_dormancy;
-	enum ofono_radio_access_mode pending_mode;
+	unsigned int pending_mode;
 	enum ofono_radio_band_gsm pending_band_gsm;
 	enum ofono_radio_band_umts pending_band_umts;
 	ofono_bool_t fast_dormancy_pending;
@@ -60,7 +60,7 @@ struct ofono_radio_settings {
 	struct ofono_atom *atom;
 };
 
-static const char *radio_access_mode_to_string(enum ofono_radio_access_mode m)
+static const char *radio_access_mode_to_string(unsigned int m)
 {
 	switch (m) {
 	case OFONO_RADIO_ACCESS_MODE_ANY:
@@ -79,11 +79,14 @@ static const char *radio_access_mode_to_string(enum ofono_radio_access_mode m)
 	if (m == (OFONO_RADIO_ACCESS_MODE_LTE|OFONO_RADIO_ACCESS_MODE_UMTS))
 		return "lte,umts";
 
+	if (m == (OFONO_RADIO_ACCESS_MODE_LTE|OFONO_RADIO_ACCESS_MODE_GSM))
+		return "lte,gsm";
+
 	return NULL;
 }
 
 static gboolean radio_access_mode_from_string(const char *str,
-					enum ofono_radio_access_mode *mode)
+					unsigned int *mode)
 
 {
 	if (g_str_equal(str, "any")) {
@@ -104,6 +107,9 @@ static gboolean radio_access_mode_from_string(const char *str,
 	} else if (g_str_equal(str, "lte,umts")) {
 		*mode = OFONO_RADIO_ACCESS_MODE_LTE|OFONO_RADIO_ACCESS_MODE_UMTS;
 		return TRUE;
+	} else if (g_str_equal(str, "lte,gsm")) {
+		*mode = OFONO_RADIO_ACCESS_MODE_LTE|OFONO_RADIO_ACCESS_MODE_GSM;
+		return TRUE;
 	}
 
 	return FALSE;
@@ -373,7 +379,7 @@ static void radio_band_set_callback(const struct ofono_error *error,
 }
 
 static void radio_set_rat_mode(struct ofono_radio_settings *rs,
-				enum ofono_radio_access_mode mode)
+				unsigned int mode)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path;
@@ -522,8 +528,7 @@ static void radio_query_band(struct ofono_radio_settings *rs)
 }
 
 static void radio_rat_mode_query_callback(const struct ofono_error *error,
-					enum ofono_radio_access_mode mode,
-					void *data)
+						int mode, void *data)
 {
 	struct ofono_radio_settings *rs = data;
 	DBusMessage *reply;
@@ -588,7 +593,7 @@ static DBusMessage *radio_set_property(DBusConnection *conn, DBusMessage *msg,
 
 	if (g_strcmp0(property, "TechnologyPreference") == 0) {
 		const char *value;
-		enum ofono_radio_access_mode mode;
+		unsigned int mode;
 
 		if (rs->driver->set_rat_mode == NULL)
 			return __ofono_error_not_implemented(msg);
